﻿/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "lpc17xx_pinsel.h"
#include "lpc17xx_timer.h"
#include "lpc17xx_ssp.h"
#include "lpc17xx_i2c.h"
#include "lpc17xx_gpio.h"
#include "lpc17xx_clkpwr.h"

#include "SEC_AOAA.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/

//Pin 12 on Serial Expansion Connector
#define RST_DIR_OUT			SEC14_PIN12_SET_OUTP
#define RST_SET_HIGH		SEC14_PIN12_SET_HIGH
#define RST_SET_LOW			SEC14_PIN12_SET_LOW

//Pin 14 on Serial Expansion Connector
#define DISCHARGE_DIR_OUT	SEC14_PIN14_SET_OUTP
#define DISCHARGE_SET_HIGH	SEC14_PIN14_SET_HIGH
#define DISCHARGE_SET_LOW	SEC14_PIN14_SET_LOW

//Pin 6 on Serial Expansion Connector
#define CS_DIR_OUT			SEC14_PIN6_SET_OUTP
#define CS_SET_HIGH			SEC14_PIN6_SET_HIGH
#define CS_SET_LOW			SEC14_PIN6_SET_LOW

//Pin 13 on Serial Expansion Connector
#define PANELON_DIR_OUT		SEC14_PIN13_SET_OUTP
#define PANELON_SET_HIGH	SEC14_PIN13_SET_HIGH
#define PANELON_SET_LOW		SEC14_PIN13_SET_LOW

//Pin 6 on Serial Expansion Connector
#define DRIVERBUSY_DIR_IN	SEC14_PIN7_SET_INP
#define DRIVERBUSY_GET		SEC14_PIN7_INP

//Not implemented
#define BORDER_DIR_OUT      SEC14_PIN8_SET_OUTP
#define BORDER_SET_HIGH     SEC14_PIN8_SET_HIGH
#define BORDER_SET_LOW      SEC14_PIN8_SET_LOW

//Pin 11 on Serial Expansion Connector
#define PWM_DIR_OUT			SEC14_PIN11_SET_OUTP
#define PWM_SET_HIGH		SEC14_PIN11_SET_HIGH
#define PWM_SET_LOW			SEC14_PIN11_SET_LOW


/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/
#define SSP_CLOCK 1000000	//10MHz SPI clock

#define I2CDEV LPC_I2C0
#define LM75A_I2C_ADDR 0x49 //(0x49 << 1)
#define LM75A_CMD_TEMP 0x00

/******************************************************************************
 * Local variables
 *****************************************************************************/
static void (*timerCallb)(void) = NULL;
static uint32_t msTicks = 0;
//static uint8_t ms100 = 0;

/******************************************************************************
 * Local functions
 *****************************************************************************/
static int I2CRead(uint8_t addr, uint8_t* buf, uint32_t len)
{
	I2C_M_SETUP_Type rxsetup;

	rxsetup.sl_addr7bit = addr;
	rxsetup.tx_data = NULL;	// Get address to read at writing address
	rxsetup.tx_length = 0;
	rxsetup.rx_data = buf;
	rxsetup.rx_length = len;
	rxsetup.retransmissions_max = 3;

	if (I2C_MasterTransferData(I2CDEV, &rxsetup, I2C_TRANSFER_POLLING) == SUCCESS){
		return (0);
	} else {
		return (-1);
	}
}

static int I2CWrite(uint8_t addr, uint8_t* buf, uint32_t len)
{
	I2C_M_SETUP_Type txsetup;

	txsetup.sl_addr7bit = addr;
	txsetup.tx_data = buf;
	txsetup.tx_length = len;
	txsetup.rx_data = NULL;
	txsetup.rx_length = 0;
	txsetup.retransmissions_max = 3;

	if (I2C_MasterTransferData(I2CDEV, &txsetup, I2C_TRANSFER_POLLING) == SUCCESS){
		return (0);
	} else {
		return (-1);
	}
}

static int32_t lm75a_readTemp(void)
{
    uint8_t temp[2];
    int16_t t = 0;

    I2CWrite(LM75A_I2C_ADDR, LM75A_CMD_TEMP, 1);
    I2CRead(LM75A_I2C_ADDR, temp, 2);

    /* 11 MSB bits used. Celcius is calculated as Temp data * 1/8 */
    t = ((temp[0] << 8) | (temp[1]));

    return ((t * 100) >> 8);
}

/******************************************************************************
 *
 * Description:
 *   Initialize I2C0
 *
 *****************************************************************************/
static void i2c0_init(void)
{
	PINSEL_CFG_Type PinCfg;

	/* Initialize I2C0 pin connect */
	PinCfg.Funcnum = 1;
	PinCfg.Portnum = 0;
	PinCfg.Pinnum = 27;
	PINSEL_ConfigPin(&PinCfg);
	PinCfg.Pinnum = 28;
	PINSEL_ConfigPin(&PinCfg);

	// Initialize I2C0 peripheral
	I2C_Init(LPC_I2C0, 100000);

	/* Enable I2C0 operation */
	I2C_Cmd(LPC_I2C0, ENABLE);
}

/******************************************************************************
 * Public functions
 *****************************************************************************/
uint32_t bsp_getMsTicks(void)
{
  return msTicks;
}

void SysTick_Handler(void)
{
	msTicks++;
	if ((timerCallb != NULL) && ((msTicks % 1000) == 0))
	{
//		msTicks = 0;
		timerCallb();
	}
}

void bsp_spiInit(void)
{
	SSP_CFG_Type sspCfg;

	SEC14_INIT_SPI;

	// initialize SSP configuration structure to default
	SSP_ConfigStructInit(&sspCfg);

	// set clock rate
	sspCfg.ClockRate = SSP_CLOCK;

	// set number of databits
	sspCfg.Databit = SSP_DATABIT_8;

	// Initialize SSP peripheral with parameter given in structure above
	SSP_Init(SSP_PORT, &sspCfg);

	// Enable SSP peripheral
	SSP_Cmd(SSP_PORT, ENABLE);
}

/******************************************************************************
 *
 * Description:
 *   BSP initialize function. Must be called before any other BSP related
 *   functions.
 *
 *****************************************************************************/
void bsp_init(void)
{
//	GPIOInit();
	// SW2
    GPIO_SetDir( 2, (1<<11), 0 );

    RST_DIR_OUT;
	DISCHARGE_DIR_OUT;
	CS_DIR_OUT;
	PANELON_DIR_OUT;
	DRIVERBUSY_DIR_IN;
	BORDER_DIR_OUT;

	bsp_spiInit();

	/* Initialize I2C and LM75 */
	i2c0_init();
	lm75a_readTemp();

  // RGB LED6
  GPIO_SetDir( 2, (1<<0), 1 );
  GPIO_SetDir( 2, (1<<1), 1 );
  GPIO_SetDir( 2, (1<<2), 1 );
  // RGB LED7
  GPIO_SetDir( 2, (1<<3), 1 );
  GPIO_SetDir( 2, (1<<4), 1 );
  GPIO_SetDir( 2, (1<<5), 1 );
  // turn leds off
  GPIO_SetValue( 2, (1<<0|1<<1|1<<2|1<<3|1<<4|1<<5));
}

void bsp_spiWrite(uint8_t* data, uint16_t len)
{
  SSP_DATA_SETUP_Type sspCfg;

  sspCfg.tx_data = data;
  sspCfg.rx_data = NULL;
  sspCfg.length  = len;
  SSP_ReadWrite (SSP_PORT, &sspCfg, SSP_TRANSFER_POLLING);
}

void bsp_spiRead(uint8_t* data, uint16_t len)
{
  SSP_DATA_SETUP_Type sspCfg;

  sspCfg.rx_data = data;
  sspCfg.tx_data = NULL;
  sspCfg.length  = len;
  SSP_ReadWrite (SSP_PORT, &sspCfg, SSP_TRANSFER_POLLING);
}

/******************************************************************************
 *
 * Description:
 *   Start a timer at specified frequency.
 *
 * Params:
 *   [in] callb - callback function called at given frequency
 *
 *****************************************************************************/
void bsp_startTimer(void (*callb)(void))
{
  timerCallb = callb;
  SysTick_Config(SystemCoreClock/1000);
}

/******************************************************************************
 *
 * Description:
 *   Delay specified number of milliseconds
 *
 * Params:
 *   [in] ms - number of milliseconds to delay
 *
 *****************************************************************************/
void bsp_delayMs(uint32_t ms)
{
  Timer0_Wait(ms);
}

/******************************************************************************
 *
 * Description:
 *   Delay specified number of microseconds
 *
 * Params:
 *   [in] us - number of microseconds to delay
 *
 *****************************************************************************/
void bsp_delayUs(uint32_t us)
{
  Timer0_us_Wait(us);
}

/******************************************************************************
 *
 * Description:
 *   Read temperature
 *
 * Params:
 *   [out] temperature - (int16_t) temperature in Celsius
 *
 *****************************************************************************/
int16_t bsp_getTemp(void)
{
	lm75a_readTemp();
	Timer0_Wait(5);
	return (int16_t)(lm75a_readTemp()/100);
}

/******************************************************************************
 *
 * Description:
 *   Read temperature
 *
 * Params:
 *   [out] temperature - (int16_t) temperature * 100
 *
 *****************************************************************************/
uint8_t bsp_getKey(void)
{
	if ((GPIO_ReadValue(2) & (1<<11)) == 0)
		return KEY_PRESS;
	return KEY_NO_PRESS;
}

/******************************************************************************
 *
 * Description:
 *   Set direction of a PIN
 *
 * Params:
 *   [in] pin - PIN number according to SEC numbering (1-14)
 *   [in] dir - set 1 for output and 0 for input
 *
 *****************************************************************************/
void bsp_pinDir(uint8_t pin, uint8_t dir)
{
  switch(pin) {

  case 6:
    // p0.6
    GPIO_SetDir(0, (1<<6), dir);

    break;

  case 7:
    // p4.29
    GPIO_SetDir(4, (1<<29), dir);

    break;
  case 8:
    // p4.28
    GPIO_SetDir(4, (1<<28), dir);

    break;
  case 11:
    // p3.25
    GPIO_SetDir(3, (1<<25), dir);

    break;
  case 12:
    // p3.26
    GPIO_SetDir(3, (1<<26), dir);

    break;
  case 13:
    // p0.23
    GPIO_SetDir(0, (1<<23), dir);

    break;
  case 14:
    // p0.26
    GPIO_SetDir(0, (1<<26), dir);

    break;
  default:
    // ignore other pin values
    break;
  }
}

/******************************************************************************
 *
 * Description:
 *   Set PIN output high
 *
 * Params:
 *   [in] pin - PIN number according to SEC numbering (1-14)
 *
 *****************************************************************************/
void bsp_pinSet(uint8_t pin)
{
  switch(pin) {

  case 6:
    GPIO_SetValue(0, (1<<6));
    break;

  case 7:
    GPIO_SetValue(4, (1<<29));
    break;
  case 8:
    GPIO_SetValue(4, (1<<28));
    break;
  case 11:
    GPIO_SetValue(3, (1<<25));
    break;
  case 12:
    GPIO_SetValue(3, (1<<26));
    break;
  case 13:
    GPIO_SetValue(0, (1<<23));
    break;
  case 14:
    GPIO_SetValue(0, (1<<26));
    break;
  default:
    // ignore other pin values
    break;
  }
}

/******************************************************************************
 *
 * Description:
 *   Set PIN output low
 *
 * Params:
 *   [in] pin - PIN number according to SEC numbering (1-14)
 *
 *****************************************************************************/
void bsp_pinClr(uint8_t pin)
{
  switch(pin) {

  case 6:
    GPIO_ClearValue(0, (1<<6));
    break;

  case 7:
    GPIO_ClearValue(4, (1<<29));
    break;
  case 8:
    GPIO_ClearValue(4, (1<<28));
    break;
  case 11:
    GPIO_ClearValue(3, (1<<25));
    break;
  case 12:
    GPIO_ClearValue(3, (1<<26));
    break;
  case 13:
    GPIO_ClearValue(0, (1<<23));
    break;
  case 14:
    GPIO_ClearValue(0, (1<<26));
    break;
  default:
    // ignore other pin values
    break;
  }
}

/******************************************************************************
 *
 * Description:
 *   Returns the state of the specified pin (1=High, 0 = Low)
 *
 * Params:
 *   [in] pin - PIN number according to SEC numbering (1-14)
 *
 *****************************************************************************/
uint8_t bsp_pinState(uint8_t pin)
{
  uint8_t result = 0;

  switch(pin) {

  case 6:
    result = ((GPIO_ReadValue(0) & (1<<6)) != 0);
    break;
  case 7:
    result = ((GPIO_ReadValue(4) & (1<<29)) != 0);
    break;
  case 8:
    result = ((GPIO_ReadValue(4) & (1<<28)) != 0);
    break;
  case 11:
    result = ((GPIO_ReadValue(3) & (1<<25)) != 0);
    break;
  case 12:
    result = ((GPIO_ReadValue(3) & (1<<26)) != 0);
    break;
  case 13:
    result = ((GPIO_ReadValue(0) & (1<<23)) != 0);
    break;
  case 14:
    result = ((GPIO_ReadValue(0) & (1<<26)) != 0);
    break;
  default:
    // ignore other pin values
    break;
  }

  return result;
}

/******************************************************************************
 *
 * Description:
 *   Turn a LED on or off
 *
 * Params:
 *   [in] on - 1 if the LED should be turned on; otherwise 0.
 *
 *****************************************************************************/
void bsp_setLed(uint8_t on)
{

  //
  // LED6 on the AOAA board
  //

  if (on) {
    // turning on the RED LED
    GPIO_ClearValue( 2, (1<<0) );
  }
  else {
    GPIO_SetValue( 2, (1<<0));
  }
}


/******************************************************************************
 *
 * Description:
 *   Turn the specified LED on or off
 *
 * Params:
 *   [in] on - 1 if the LED should be turned on; otherwise 0.
 *
 *****************************************************************************/
void bsp_setRgbLed(rgbled_t led, uint8_t on)
{
	switch (led) {
	case LED6_RED:
		if (on) {
			GPIO_ClearValue( 2, (1<<0) );
		} else {
			GPIO_SetValue( 2, (1<<0) );
		}
		break;

	case LED6_GREEN:
		if (on) {
			GPIO_ClearValue( 2, (1<<2) );
		} else {
			GPIO_SetValue( 2, (1<<2) );
		}
		break;

	case LED6_BLUE:
		if (on) {
			GPIO_ClearValue( 2, (1<<1) );
		} else {
			GPIO_SetValue( 2, (1<<1) );
		}
		break;

	case LED7_RED:
		if (on) {
			GPIO_ClearValue( 2, (1<<3) );
		} else {
			GPIO_SetValue( 2, (1<<3) );
		}
		break;

	case LED7_GREEN:
		if (on) {
			GPIO_ClearValue( 2, (1<<5) );
		} else {
			GPIO_SetValue( 2, (1<<5) );
		}
		break;

	case LED7_BLUE:
		if (on) {
			GPIO_ClearValue( 2, (1<<4) );
		} else {
			GPIO_SetValue( 2, (1<<4) );
		}
		break;
	}
}

